/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.vfs.impl;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.tinygroup.vfs.FileObject;
import org.tinygroup.vfs.SchemaProvider;

public class JarFileObject implements FileObject {

	private FileObject parent;
	private List<FileObject> children;
	private SchemaProvider schemaProvider;
	private ZipFile jarFile = null;
	private File file = null;
	private ZipEntry ZipEntry = null;
	private static final String JAR_PROTOCAL = "jar:";
	private static final String ZIP_PROTOCAL = "zip:";
	private static final String FILE_PROTOCAL = "file:";
	private File cacheFile = null;

	public JarFileObject(SchemaProvider schemaProvider, String resource) {
		this.schemaProvider = schemaProvider;
		try {
			if (resource.startsWith(JAR_PROTOCAL)
					|| resource.startsWith(ZIP_PROTOCAL)) {
				resource = resource.substring(JAR_PROTOCAL.length());
			} else if (resource.startsWith(FILE_PROTOCAL)) {
				resource = resource.substring(FILE_PROTOCAL.length());
			}
			if (resource.startsWith(FILE_PROTOCAL)) {
				resource = resource.substring(FILE_PROTOCAL.length());
			}
			this.file = new File(resource);
			if (file.exists()) {
				this.jarFile = new ZipFile(resource);
			}
		} catch (IOException e) {
			throw new RuntimeException(resource + "打开失败，错误：" + e.getMessage());
		}
	}

	public JarFileObject(JarFileObject parent, ZipEntry entry) {
		this.schemaProvider = parent.getSchemaProvider();
		this.jarFile = parent.jarFile;
		this.file = parent.file;
		this.ZipEntry = entry;
	}

	public String getFileName() {
		if (ZipEntry != null) {
			String[] names = ZipEntry.getName().split("/");
			return names[names.length - 1];
		} else {
			return file.getName();
		}
	}

	public String getPath() {
		if (ZipEntry != null) {
			return "/" + ZipEntry.getName();
		}
		return "/";
	}

	public String getAbsolutePath() {
		String path = file.getAbsolutePath();
		if (File.separatorChar == '\\') {
			path = path.replaceAll("\\\\", "/");
		}
		if (ZipEntry != null) {
			path = path + "!/" + ZipEntry.getName();
		}
		return path;
	}

	public String getExtName() {
		String name;
		if (ZipEntry != null) {
			if (!ZipEntry.isDirectory()) {
				name = ZipEntry.getName();
			} else {
				return null;
			}
		} else {
			name = file.getName();
		}
		int lastIndexOfDot = name.lastIndexOf(".");
		if (lastIndexOfDot == -1) {
			// 如果不存在
			return null;
		}
		return name.substring(lastIndexOfDot + 1);
	}

	public long getSize() {
		if (ZipEntry != null) {
			return ZipEntry.getSize();
		}
		if (file.exists() && file.isFile()) {
			return file.length();
		}
		return 0;
	}

	public InputStream getInputStream() {
		try {
			if (ZipEntry != null) {
				if (cacheFile == null || !cacheFile.exists()
						|| cacheFile.length() != this.getSize()) {
					String tempPath = System.getProperty("java.io.tmpdir");
					if (!tempPath.endsWith(File.separator)) {
						tempPath = tempPath + File.separator;
					}
					tempPath = tempPath + getExtName() + File.separator;
					File tempPathFile = new File(tempPath);
					if (!tempPathFile.exists()) {
						tempPathFile.mkdirs();
					}
					cacheFile = new File(tempPath + getFileName() + "_"
							+ getLastModifiedTime());

					FileOutputStream out = new FileOutputStream(cacheFile);
					byte[] buf = new byte[(int) getSize()];
					BufferedInputStream bufferedInputStream = new BufferedInputStream(
							jarFile.getInputStream(ZipEntry));
					bufferedInputStream.read(buf);
					bufferedInputStream.close();
					out.write(buf);
					out.close();
				}
				return new BufferedInputStream(new FileInputStream(cacheFile));
			}
			if (file.exists() && file.isFile()) {
				return new BufferedInputStream(new FileInputStream(file));
			} else {
				throw new RuntimeException(file.getAbsolutePath()
						+ "不存在，或不是文件。");
			}
		} catch (Exception e) {
			throw new RuntimeException(file.getAbsolutePath()
					+ "获取FileInputStream出错，原因" + e);
		} finally {
		}
	}

	public String toString() {
		return getURL().toString();
	}

	public boolean isFolder() {
		if (ZipEntry != null) {
			return ZipEntry.isDirectory();
		}
		if (file.exists()) {
			return true;
		}
		return false;
	}

	public FileObject getParent() {
		return parent;
	}

	public List<FileObject> getChildren() {
		if (!file.exists()) {
			return null;
		}
		if (children == null) {
			children = new ArrayList<FileObject>();
			Enumeration<ZipEntry> e = (Enumeration<ZipEntry>) jarFile.entries();
			while (e.hasMoreElements()) {
				ZipEntry entry = (ZipEntry) e.nextElement();
				if (parent == null) {
					String[] names = entry.getName().split("/");
					// 如果当前是jar文件，如果
					if (names.length == 1) {
						addSubItem(entry);
					}
				} else {// 如果不是根目录
					String parentName = ZipEntry.getName();
					if (!entry.getName().equals(ZipEntry.getName())
							&& entry.getName().startsWith(parentName)) {
						String fn = entry.getName().substring(
								parentName.length());
						String[] names = fn.split("/");
						if (names.length == 1) {
							addSubItem(entry);
						}
					}
				}
			}
		}

		return children;
	}

	private void addSubItem(ZipEntry entry) {
		JarFileObject jarFileObject = new JarFileObject(this, entry);
		jarFileObject.parent = this;
		children.add(jarFileObject);
	}

	public long getLastModifiedTime() {
		if (ZipEntry != null) {
			return ZipEntry.getTime();
		}
		return file.lastModified();
	}

	public SchemaProvider getSchemaProvider() {
		return schemaProvider;
	}

	public boolean isExist() {
		if (ZipEntry != null) {
			return true;
		}
		return file.exists();
	}

	public boolean isInPackage() {
		return ZipEntry != null;
	}

	public FileObject getChild(String fileName) {
		if (getChildren() != null) {
			for (FileObject fileObject : getChildren()) {
				if (fileObject.getFileName().equals(fileName)) {
					return fileObject;
				}
			}
		}
		return null;
	}

	public URL getURL() {
		try {
			if (ZipEntry != null) {
				return new URL(JAR_PROTOCAL + FILE_PROTOCAL + getAbsolutePath());
			} else {
				return new URL(FILE_PROTOCAL + getAbsolutePath());
			}
		} catch (MalformedURLException e) {
			throw new RuntimeException(e);
		}
	}

	public OutputStream getOutputStream() {
		throw new UnsupportedOperationException();
	}

	public void setParent(FileObject fileObject) {
		this.parent = fileObject;

	}
}
